#!/usr/bin/perl
#
# Clean Route Waypoint names and symbols                         V1.05
# Reorder waypoints and clean symbols in GPX route files.
# Syntax:
#    clnrte [options] infile1.gpx [infile2.gpx ...[infileN.gpx]]
#    Where infile1.gpx ... infileN.gpx are route files in GPX format.
# Options:
#    --help or -h- Display this message.
#    --outfile or -o - Output File (optional), default is infileX_cln.gpx,
#                      (does not work with multiple input files).
#    --reorder or -r | --noreorder or -nor - Reorder/Don't reorder waypoint
#                                            names (default is to reorder)
#    --start or -s # - Starting waypoint number, if reordering, (default is 0).
#    --digits or -d # - Minimum number of digits in waypoint name, if reordering
#                       (default is 3).
#    --cleansym or -c | --nocleansym or -noc - Clean/Don't clean waypoint
#                                       symbols (default is to clean symbols).
#    --wpsym or -w 'string' - Waypoint symbol, if cleaning symbols, 
#                             (default is 'Waypoint').
#    --verbose or -v | --noverbose or -nov - Verbose/Nonverbose mode.
#
# Copyright (C) Ron Olson 2014

use Getopt::Long;
my $version = "1.05";
my $help;
my $start = '';
my $verbose;
my $outfile = '';
my $reorder = 1;
my $cleansym = 1;
my $digits = 3;
my $wpsym = 'Waypoint';
my $overwrite = 0;

print stderr "\nclnrte (Clean Route) Version: $version\n",
"Copyright (C) Ron Olson 2014\n\n";

GetOptions ("help" => \$help,
            "start=i" => \$start,
            "verbose!" => \$verbose,
            "outfile=s" => \$outfile,
            "reorder!" => \$reorder,
            "cleansym!" => \$cleansym, 
            "digits=i" => \$digits,
            "wpsym=s" => \$wpsym) 
or die("Error in command line arguments\n");

if ($help) {
   print stderr 
   "Reorder waypoints and clean symbols in GPX route files.\n" .
   "Syntax:\n" .
   "   clnrte [options] infile1.gpx [infile2.gpx ...[infileN.gpx]]\n" .
   "   Where infile1.gpx ... infileN.gpx are route files in GPX format.\n" .
   "Options:\n" .
   "   --help or -h - Display this message.\n".
   "   --outfile or -o - Output File (optional), default is infileX_cln.gpx,\n" .
   "                     (does not work with multiple input files).\n" .
   "   --reorder or -r | --noreorder or -nor - Reorder | Don't reorder\n" .
   "                                  waypoint names (default is to reorder)\n" .
   "   --start or -s # - Starting waypoint number, if reordering, (default is 0).\n" .
   "   --digits or -d # - Minimum number of digits in waypoint name, if reordering\n" .
   "                      (default is 3).\n".
   "   --cleansym or -c | --nocleansym or -noc - Clean | Don't clean waypoint\n" .
   "                                      symbols (default is to clean symbols).\n" .
   "   --wpsym or -w 'string' - Waypoint symbol, if cleaning symbols, \n".
   "                            (default is 'Waypoint').\n" .
   "   --verbose or -v | --noverbose or -nov - Verbose | Nonverbose mode.\n";
   exit;
}

if ($verbose) {
   print stderr "clnrte.pl ";
   if ($outfile) {
      print stderr "--outfile=\"$outfile\" ";
   }
   if ($reorder) {
      print stderr "--reorder --start $start --digits $digits ";
   } else {
      print stderr "--noreorder ";
   }
   if ($cleansym) {
      print stderr "--cleansym --wpsym \"$wpsym\" ";
   } else {
      print stderr "--nocleansym ";
   }
   if ($verbose) {
      print stderr "--verbose ";
   } else {
      print stderr "--noverbose ";
   }
   foreach my $arg (@ARGV) {
      print stderr "   \"$arg\" ";
   }
   print stderr "\n";
}

die ("Error: Must specify valid GPX route file.") if (@ARGV == 0);

if (($outfile ne '') and (@ARGV > 1)) {
   die ("Error: Multiple route files not allowed with --outfile option.");
}

if ($start eq '' && $reorder) {
   $start = 0;
   print stderr "Please enter a waypoint starting number-> ";
   $start = <stdin>;
}

my $wptnm = $start;
my $clnnum = 0;

foreach my $infile (@ARGV) {

   print stderr "infile=\"$infile\"\n" if ($verbose);
   open INP, $infile or die("Error opening \"$infile\"");

   if (-e $infile) {
      if (-r $infile) {
         print stderr "File \"$infile\" is readable\n" if ($verbose);
      } else {
         print stderr "Error: File \"$infile\" is not readable\n";
         exit;
      }
   } else {
      print stderr "Error: File \"$infile\" does not exist\n";
      exit;
   }

   $infile =~ /^(.+?)\.(.*)/i;
   my $fname = $1;
   print "Input file name is \"$fname\"\n" if ($verbose);
   my $ftype = $2;
   print "Input file type is \"$ftype\"\n" if ($verbose);
   if (lc$ftype ne 'gpx') {
      print stderr "Warning: Input file is not type GPX\n";
      my $ans = '';
      do {
         print stderr "Continue with \"$infile\"? (Yes/No): ";
         $ans = <stdin>;
         exit if ($ans =~ /No?/i);
      } while (!($ans =~ /Ye?s?/i));
   }

   if ($outfile eq '') {
      if ($ftype eq '') {
         $ftype = 'gpx';
      }
      $outfile = $fname."_cln.$ftype";
   }

   print stderr "outfile=\"$outfile\"\n" if ($verbose);

   if (-e $outfile) {
      if (-w $outfile) {
         print stderr "Warning: Output file \"$outfile\" already exists.\n";
         my $ans = '';
         while ((!$overwrite) and !($ans =~ /Ye?s?/i)) {
            print stderr "Overwrite \"$outfile\"? (Yes/No/All): ";
            $ans = <stdin>;
	    if ($ans =~ /Al?l?/i) {
	       $overwrite = 1;
	    }
            exit if ($ans =~ /No?/i);
         }
      } else {
         print stderr "Error: Output file \"$outfile\" is not writable\n";
         exit;
      }
   } else {
      print stderr "File \"$outfile\" is writable\n" if ($verbose);
   }

   open OUTFILE, ">", $outfile or die "Open $outfile: $!";

   # Process input file.
   while (<INP>) {
      if ($_ =~ /\<rtept/) {
         print OUTFILE $_;
         my $rteln = <INP>;
         while (!($rteln =~ /<\/rtept>/)) {
            if ($reorder) {
               if ($rteln =~ /^(.*)\<name\>(.*)<\/name\>(.*)$/) {
                  $rteln = sprintf("$1<name>%0.".$digits."d</name>$3\n",
                                                                    $wptnm++);
               }
            }
            if ($cleansym) {
               if ($rteln =~ /^(.*)\<sym\>(.*)<\/sym\>(.*)$/) {
                  $rteln = "$1<sym>$wpsym</sym>$3\n";
                  $clnnum++;
               }
            }
            # Don't show waypoint name
            $rteln =~ s/viz_name\>1/viz_name>0/;
            print OUTFILE $rteln;
            $rteln = <INP>;
         }
         print OUTFILE $rteln;
      } else {
         print OUTFILE $_;
      }
   }

   close INP or die $!;
   close OUTFILE or die $!;

   print stderr "\"$infile\" processed to \"$outfile\"\n";
   $outfile = '';
}
if ($verbose) {
   my $num = $wptnm - $start;
   print stderr "$num waypoints reordered\n";
   print stderr "$clnnum waypoint symbols cleaned\n";
}
